BagCounter = class( Turbine.UI.Window );

function BagCounter:Constructor()
	Turbine.UI.Window.Constructor( self );
	self:InitDefaults();
	self:LoadSettings();
	self:InitWindow();
	self:InitBackpack();
	self:SetVisible(true);
end
function BagCounter:Destroy()
    self:SetVisible(false);
    self:SetWantsUpdates(false);
end
function BagCounter:Update(args)
	if self.BagStatusOld~=nil and self.LastUpdate~=nil then
		local now=Turbine.Engine.GetGameTime();
		if self.BagStatusOld~=self.BagStatus:GetText() then
			self.BagStatusOld=self.BagStatus:GetText();
			self:SetWantsUpdates(true);
		elseif Turbine.Engine.GetGameTime()-self.LastUpdate>2 then
			self.BGImage:SetBackColor(Turbine.UI.Color(0,0,0,0));
			self.LastUpdate=nil;
			self.BagStatusOld=nil;
			self:SetWantsUpdates(false);
		else
			local alpha=self.BGImage:GetBackColor().A;
			if (self.BGImage.FadeDirection>0 and alpha<0.8) or (self.BGImage.FadeDirection<0 and alpha>0.1) then
				self.BGImage:SetBackColor(Turbine.UI.Color(alpha+self.BGImage.FadeDirection,1,1,0));
				self:SetWantsUpdates(true);
			else
				self.BGImage.FadeDirection=-self.BGImage.FadeDirection
			end
		end
	end
end
function BagCounter:BackPackRefresh(ignoreIndex )
	local size=BackPack:GetSize();
	local count=0;
	for i=1,size,1 do
		if BackPack:GetItem(i)~=nil then 
			if ignoreIndex==nil or ignoreIndex~=i then count=count+1 end
		end
	end
	local txtLabel=count.."/"..size;
	self.BagStatusOld=self.BagStatus:GetText();
	self.BagStatus:SetText(txtLabel);
	self.LastUpdate=Turbine.Engine.GetGameTime();
	self:SetWantsUpdates(true);
end
function BagCounter:ItemAdded( sender, args )
	self:BackPackRefresh();
end
function BagCounter:ItemMoved( sender, args )
	self:BackPackRefresh();
end
function BagCounter:ItemRemoved( sender, args )
	self:BackPackRefresh(args.Index); --workaround as BackPack still remembers the item at this position..
end
function BagCounter:SizeChanged( sender, args )
	self:BackPackRefresh();
end
function BagCounter:InitBackpack()
	BackPack.AddListener(self);
	self:BackPackRefresh();
end
function BagCounter:InitWindow()
	self:SetSize(80,16);
	self:SetPosition(self.Settings.BagCountPosX,self.Settings.BagCountPosY)
	self:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.BGImage=Turbine.UI.Control();
	self.BGImage.FadeDirection=0.02;
	self.BGImage:SetParent(self);
	self.BGImage:SetPosition(0,0);
	self.BGImage:SetSize(self:GetWidth(),self:GetHeight());
	self.BGImage:SetBackground(self.Resource.BagCounterBg);
	self.BGImage:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay);
	self.BagImage=Turbine.UI.Control();
	self.BagImage:SetParent(self.BGImage);
	self.BagImage:SetSize(14,16);
	self.BagImage:SetPosition(self.BGImage:GetWidth()-self.BagImage:GetWidth(),0);
	self.BagImage:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.BagImage:SetBackground(self.Resource.Bag);
	self.BagStatus=Turbine.UI.Label();
	self.BagStatus:SetParent(self.BGImage);
	self.BagStatus:SetPosition(0,0);
	self.BagStatus:SetSize(self.BGImage:GetWidth()-self.BagImage:GetWidth()-3,self.BGImage:GetHeight());
	self.BagStatus:SetFont(Turbine.UI.Lotro.Font.Verdana14);
	self.BagStatus:SetFontStyle(Turbine.UI.FontStyle.Outline);
	self.BagStatus:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleRight );
	self.BagStatus:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.BagStatus:SetForeColor(Turbine.UI.Color(0.8,1,1,1));
	self.BagStatus:SetOutlineColor(Turbine.UI.Color(0.5,0,0,0));
	self.BagStatus:SetText("");
	--lock and unlock UI movement
	self.UILocked=true; --todo: something like "Turbine.UI.Lotro.LotroUI:IsEnabled();"
	if not self.UILocked then
		self.dragBar = DragBar(self, "BagCounter");
		self.dragBar:SetBarOnTop(true);
		self.dragBar:SetBarVisible(true);
		self.dragBar:SetDraggable(true);
	end
    self.DragEnd = function(sender)
			local x,y=self:GetPosition();
			self.Settings.BagCountPosX=x;
			self.Settings.BagCountPosY=y;
			self:SaveSettings();
		end
	self:SetWantsKeyEvents(true);
	self.KeyDown = function(sender, args)
		if (args.Action == 0x1000007B) then -- unlock
			self.UILocked=not self.UILocked;
			if (not self.UILocked) then
				self.dragBar = DragBar(self, "BagCounter");
				self.dragBar:SetBarOnTop(true);
				self.dragBar:SetBarVisible(true);
				self.dragBar:SetDraggable(true);
			else
				if (self.dragBar ~= nil) then
					self.dragBar:SetDraggable(false);
					self.dragBar:SetTarget(nil);
					self.dragBar = nil;
				end
			end
		elseif (args.Action == 0x100000B3) then -- hide ui elements
			self:SetVisible(not self:IsVisible())
		end
	end
end
function BagCounter:InitDefaults()
	self.Settings={
		};
	self.Defaults={
		 BagCountPosX=Turbine.UI.Display.GetWidth()-80
		,BagCountPosY=Turbine.UI.Display.GetHeight()*0.667
		};
	self.Resource={
		 Bag="PChaly/ChalyInv/img/bag.tga"
		,BagCounterBg="PChaly/ChalyInv/img/BagCounterBg.tga"
		};
	self.BagStatusOld=nil;
end
function BagCounter:LoadSettings()
	local fLoad = Turbine.PluginData.Load;
	local tSettings = fLoad( Turbine.DataScope.Character, "BagCounterSettings" );
	if ( type(tSettings) ~= "table" ) then
		self.Settings = PChaly.ChalyInv.DeepCopy( self.Defaults );
	else
		if ( Turbine.Engine.GetLocale() ~= "en" ) then
			tSettings = PChaly.ChalyInv.DeepCopyDecode( tSettings );
		end
		self.Settings = tSettings;
	end
end
function BagCounter:SaveSettings()
	local fSave = Turbine.PluginData.Save; 
	local tSettings = {};
	if ( Turbine.Engine.GetLocale() ~= "en" ) then
		tSettings = PChaly.ChalyInv.DeepCopyEncode( self.Settings );
	else
		tSettings = PChaly.ChalyInv.DeepCopy( self.Settings );
	end
	fSave( Turbine.DataScope.Character, "BagCounterSettings", tSettings );
end
